/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "graph.h"
#include "compute.h"
#include "path.h"
#include "fileio.h"
#include "draw.h"
#include "move.h"
#include "ask.h"
#include "note.h"
#include "board.h"
#include "preferences.h"

#include "compress_board.h"
t_graph *main_graph;

void
on_New_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    main_graph= new_file(main_graph);
}


void
on_Open_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    open_file();
}


void
on_save_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    save_file(main_graph);
}


void
on_Save_As_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    save_as_file();
}


void
on_Merge_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    merge_file();
}


void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  if (main_graph)
    {
      t_graph *g= main_graph;
      main_graph= NULL;
      main_graph= ok_file(g);
    }
}


void
on_cancel_button1_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  if (main_graph)
    cancel_file();
}

gboolean CheckUserExit ()
{
  if (!main_graph->unsaved || ask_question(_("Modified; exit anyway?")))
    {
      if (!main_graph)
	gtk_exit (0);
      return (1);
    }
  return (0);
}

void
on_Quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (CheckUserExit())
    gtk_main_quit ();
}


void
on_Delete_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  t_node *prev;
  if (main_graph)
    {
      prev=main_graph->current_node->prev;
      if (!editable) 
	display_error (_("Please enable editable before cutting node"));
      else if (main_graph->current_node->tos==NULL && prev) 
	{
	  main_graph->unsaved= 1;
	  free_node(main_graph, main_graph->current_node);
	  main_graph->current_node= prev;
	  draw_board(main_graph, 0, 0);
	} 
      else
	display_error (_("The node is not a propper leaf"));
    }
}


void
on_auto_promote1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  auto_promote= !auto_promote;
}


void
on_Editable_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  editable= !editable;
  note_editable(editable);
}

GtkWidget *prunefactor;

void
on_prune1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    {
      if (!editable) 
	display_error (_("Please enable editable before pruning graph"));
      else
	{
	  gtk_widget_show_all (prunefactor);
	}
    }
}

void
on_button6_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  int i= -1;
  i=gnome_number_entry_get_number
    (GNOME_NUMBER_ENTRY(lookup_widget(prunefactor, "numberentry1")));
  if (i>0)
    {
      prune_graph(main_graph, main_graph->current_node, i);
      draw_board(main_graph, 0, 0);
    }
  else
    display_error ("Illegal argument to prune");
  
  gtk_widget_hide(prunefactor);
}

void
on_button8_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(prunefactor);
}

void
on_Backward_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    move_backward (main_graph, NULL);
}


void
on_Forward_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    move_forward (main_graph, NULL);
}


void
on_Start_of_Game_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    move_start (main_graph);
}


void
on_End_of_Game_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    move_end (main_graph);
}


void
on_Longest_path_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    longest_path (main_graph->current_node);
}


void
on_Longest_branch_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    longest_branch (main_graph->current_node);
}


void
on_Ordinary_path_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    ordinary_path (main_graph->current_node);
}


void
on_Flip_view_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  flip_view(main_graph);
}


void
on_Train_mode_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  train_mode= !train_mode;
  draw_board(main_graph, 0, 0);
}


void
on_Comment_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (note);
}


void
on_About_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (create_about ());
}


gboolean
on_fileselection_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide (widget);
  return TRUE;
}


void
on_ok_clicked                          (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (GTK_WIDGET(user_data));
}


gboolean
on_start_button_press_event            (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    move_start(main_graph);
  return FALSE;
}


gboolean
on_back_button_press_event             (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    move_backward(main_graph, NULL);
  return FALSE;
}


gboolean
on_forward_button_press_event          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    move_forward(main_graph, NULL);
  return FALSE;
}


gboolean
on_end_button_press_event              (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    move_end(main_graph);
  return FALSE;
}


gboolean
on_new_button_press_event              (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    main_graph= new_file(main_graph);
  return FALSE;
}


gboolean
on_open_button_press_event             (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    open_file();
  return FALSE;
}


gboolean
on_save_button_press_event             (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    save_file(main_graph);
  return FALSE;
}


gboolean
on_merge_button_press_event            (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  if (main_graph)
    merge_file();
  return FALSE;
}


void
on_app1_destroy                        (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


gboolean
on_app1_delete_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return (CheckUserExit ()==0);
}

gboolean
on_flip_button_press_event             (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  flip_view(main_graph);
  return FALSE;
}


void
on_auto_flip1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  auto_flip();
}

void
on_note_ok_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  if (main_graph && editable) 
    apply_note(main_graph);
  gtk_widget_hide (note);
}

void
on_note_apply_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  if (main_graph)
    {
      if (!editable) 
	display_error (_("Please enable editable before applying"));
      apply_note(main_graph);
    }
}


void
on_note_cancel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (note);
}

void
on_backward_branch1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    move_backward_branch(main_graph);
}


void
on_forward_branch1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    move_forward_branch(main_graph);
}


void
on_save_game1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if (main_graph)
    save_game();
}

void
on_automove_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  auto_move();
}

void
on_hide_toolbar1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *w;
  static char hide_toolbar;
  w=lookup_widget (app1, "toolbar1");
  hide_toolbar= !hide_toolbar;
  if (hide_toolbar) 
    {
      gtk_widget_hide_all(w);
    } else 
    {
      gtk_widget_show_all(w);
    }
}


void
on_hide_marker1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  mark_board_position (0, 0, FALSE);
}


void
on_moveenter_activate                  (GtkEditable     *edit,
                                        gpointer         user_data)
{
  char *move;
  t_cord from, to, extra;
  GSList *l;
  t_arc *a;
  char board[10][10];
  char board2[10][10];
#ifdef COMPRESS_BOARD
  int cboard2[9];
#endif
  move= gtk_entry_get_text(GTK_ENTRY(edit));
#ifdef COMPRESS_BOARD
  uncompress_board(board, main_graph->current_node->board);
#else
  copy_board (board, main_graph->current_node->board);
#endif
  if (!move_board(board, board2, move, &from, &to, &extra))
    {
#ifdef COMPRESS_BOARD
      compress_board (board2, cboard2);
#endif
      for (l= main_graph->current_node->tos; l; l= l->next)
	{
	  a=(t_arc *) l->data;
#ifdef COMPRESS_BOARD
	  if(boards_is_equal(cboard2, a->to_node->board))
#else
	  if(boards_is_equal(board2, a->to_node->board))
#endif
	    {
	      move_forward(main_graph, a->to_node);
	      goto found;
	    }
	}
      if (editable)
	{
	  main_graph->current_node=
	    add_one_node (main_graph, main_graph->current_node, 
		      0, 0, 0, 0, auto_promote, move, board);
	  if (auto_flip_on) 
	    flip_view(main_graph);
	  else
	    draw_board (main_graph, 0, 0);
	}
    found:;
    }
  else
    {
      /* Illegal move */
    }


  gtk_entry_set_text (GTK_ENTRY(edit), "");
}




void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (preferences);
}

void
on_propertybox1_apply                  (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
  switch (arg1)
    {
    case 0:
      preferences_set_color ();
      draw_board(main_graph, 0, 0);
      break;
    case 1:
      preferences_set_default_database ();
      break;
    case 2:
      preferences_set_auto ();
      break;
    case -1:
      gnome_config_sync ();
    }
}


void
on_colorpicker_color_set               (GnomeColorPicker *gnomecolorpicker,
                                        guint            arg1,
                                        guint            arg2,
                                        guint            arg3,
                                        guint            arg4,
                                        gpointer         user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(preferences));
}


void
on_entry_default_database_changed      (GtkEditable     *edit,
                                        gpointer         user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(preferences));
}


void
on_combo_entry_default_database_changed
                                        (GtkEditable     *edit,
                                        gpointer         user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(preferences));
}


void
on_western_pieces_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(preferences));
}


void
on_clasical_pieces_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(preferences));
}


void
on_auto_promote_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(preferences));
}


void
on_auto_move_toggled                   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  gnome_property_box_changed (GNOME_PROPERTY_BOX(preferences));
}


