/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "graph.h"
#include "note.h"
#include "compute.h"
#include "support.h"
#include "hash.h"
#include "move.h"
#include "draw.h"
#include "ask.h"
#include "fileio.h"

GtkWidget *note;

void apply_note (t_graph *g)
{
  t_data *current_data;
  GtkWidget *w;
  t_arc *a;

  
  if ((current_data=compute (g, g->current_node)->current_data))
    {
      w= lookup_widget (note, "sente");
      current_data->sente= find_string_in_hash_table
	(gtk_entry_get_text(GTK_ENTRY(w)));

      w= lookup_widget (note, "gote");
      current_data->gote= find_string_in_hash_table
	(gtk_entry_get_text(GTK_ENTRY(w)));

      w= lookup_widget (note, "date");
      current_data->date= find_string_in_hash_table
	(gtk_entry_get_text(GTK_ENTRY(w)));

      w= lookup_widget (note, "event");
      current_data->event= find_string_in_hash_table
	(gtk_entry_get_text(GTK_ENTRY(w)));

      w= lookup_widget (note, "opening");
      current_data->opening= find_string_in_hash_table
	(gtk_entry_get_text(GTK_ENTRY(w)));

      w= lookup_widget (note, "result");
      current_data->result= find_string_in_hash_table
	(gtk_entry_get_text(GTK_ENTRY(w)));
    }

  w= lookup_widget (note, "position_note");
  g->current_node->note= find_string_in_hash_table
    (gtk_editable_get_chars (GTK_EDITABLE(w), 0, 
				 gtk_text_get_length  (GTK_TEXT(w))));
  g->current_node->leaf= 
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lookup_widget (note, "leaf")));

  a= from_arc(g->current_node);
  if (a) a->bad= gtk_toggle_button_get_active 
	   (GTK_TOGGLE_BUTTON (lookup_widget (note, "bad")));
}

void note_editable(gboolean editable)
{
  gtk_entry_set_editable (GTK_ENTRY (lookup_widget (note, "sente")), editable);
  gtk_entry_set_editable (GTK_ENTRY (lookup_widget (note, "gote")), editable);
  gtk_entry_set_editable (GTK_ENTRY (lookup_widget (note, "date")), editable);
  gtk_entry_set_editable (GTK_ENTRY (lookup_widget (note, "event")), editable);
  gtk_entry_set_editable (GTK_ENTRY (lookup_widget (note, "opening")), editable);
  gtk_entry_set_editable (GTK_ENTRY (lookup_widget (note, "result")), editable);
  gtk_text_set_editable (GTK_TEXT (lookup_widget (note, "position_note")), editable);


  gtk_widget_set_sensitive (GTK_WIDGET (lookup_widget (note, "leaf")), editable);
  gtk_widget_set_sensitive (GTK_WIDGET (lookup_widget (note, "bad")), editable);

}

void display_note(t_graph *g, t_node *n)
{
  t_data *current_data;
  GtkWidget *w;
  t_arc *a= NULL;
  if (n==NULL) 
    current_data= NULL;
  else
    current_data= g->current_node->compute->current_data;
  w= lookup_widget (note, "sente");
  if (current_data && current_data->sente) 
    gtk_entry_set_text (GTK_ENTRY(w),  current_data->sente);
  else
    gtk_entry_set_text (GTK_ENTRY(w),  "");
  
  w= lookup_widget (note, "gote");
  if (current_data && current_data->gote) 
    gtk_entry_set_text (GTK_ENTRY(w),  current_data->gote);
  else
    gtk_entry_set_text (GTK_ENTRY(w),  "");

  w= lookup_widget (note, "event");
  if (current_data && current_data->event) 
    gtk_entry_set_text (GTK_ENTRY(w),  current_data->event);
  else
    gtk_entry_set_text (GTK_ENTRY(w),  "");

  w= lookup_widget (note, "date");
  if (current_data && current_data->date) 
    gtk_entry_set_text (GTK_ENTRY(w),  current_data->date);
  else
    gtk_entry_set_text (GTK_ENTRY(w),  "");

  w= lookup_widget (note, "opening");
  if (current_data && current_data->opening) 
    gtk_entry_set_text (GTK_ENTRY(w),  current_data->opening);
  else
    gtk_entry_set_text (GTK_ENTRY(w),  "");

  w= lookup_widget (note, "result");
  if (current_data && current_data->result) 
    gtk_entry_set_text (GTK_ENTRY(w),  current_data->result);
  else
    gtk_entry_set_text (GTK_ENTRY(w),  "");

  w= lookup_widget (note, "position_note");
  gtk_text_set_point (GTK_TEXT(w), 0);
  gtk_text_forward_delete (GTK_TEXT(w), gtk_text_get_length  (GTK_TEXT(w)));
  if (n && n->note)
    {
      gtk_text_insert (GTK_TEXT(w), NULL, NULL, NULL, n->note, strlen (n->note));
    }

  gtk_toggle_button_set_active 
    (GTK_TOGGLE_BUTTON (lookup_widget (note, "leaf")), n?n->leaf:FALSE);

  if (n) a= from_arc(n);
  gtk_toggle_button_set_active 
    (GTK_TOGGLE_BUTTON (lookup_widget (note, "bad")), a?a->bad:FALSE);
}
